  function [p1,p2,p3,  J1,J2,J3,  J] = boot_test_fixed_regressor(Y,X,T,R,h,kernelType,XTol,maxIter,nb)
%% This function is used to calculate the bootstrap test statistic
%% p1,p2, p3 report the bootstrap p-values for three tests using the wild bootstrap


%% function start here

n = size(X,1);
N = n/T;

%% estimate the time-varying panel using kernal approach
res = kpanelife(Y,X,T,R,h,kernelType,1,XTol,maxIter);
e = res.e; F = res.F; C = res.C; beta = res.beta; L = res.L;

%% estimate the time-invariant panel
[beta_,C_,F_,~,~,e_] = panelife(Y,X,T,R,XTol,maxIter);

J1 = test1(F,e,F_,e_,h,kernelType);

J2 = test2(X,beta,e,L,F,h,kernelType);

W = Y-sum(X.*repmat(beta,N,1),2);
[F3,~,C3] = o3(W,T,R);
J3 = test3(C,F,e,C3,F3,h,kernelType);
 
beta_bcm = mean(beta,1)';
S1 = reshape(e_,T,N)'*reshape(e_,T,N)/T; % N by N
S1 = covshrink(S1,N,0.001);
W1 = S1^(1/2);

J = zeros(nb,3);
parfor i=1:nb
 
      %  i
    % if we use the fixed regressor wild bootstrap
    ee = e_.*randn(N*T,1);
    % ee = reshape((W1*randn(N,T))',n,1);
    eTN = reshape(ee, T,N); 

    % m = mean(eTN)';
    % s = std(eTN)';
    % u = ones(T,1);
    % 
    % 
    % eeTN = (eTN - u*m')./(u*s');
    eeTN = eTN; 
    ee= reshape(eeTN, N*T,1);  

    %% generate data for H1: time-invariant factor loadings & slope 
    Y1 = X*beta_ + C_ + ee;

    res = kpanelife(Y1,X,T,R,h,kernelType,1,XTol,maxIter);
    eb1 = res.e; Fb1 = res.F; Cb1=res.C;
    [~,Cb1_,Fb1_,~,~,eb1_] = panelife(Y1,X,T,R,XTol,maxIter);

    J1b = test1(Fb1,eb1,Fb1_,eb1_,h,kernelType);

 

    %% generate data for H2: allow for TV factor loadings 

    Y2 = X*beta_bcm + C + ee;
    
    res = kpanelife(Y2,X,T,R,h,kernelType,1,XTol,maxIter);
    eb2 = res.e; Fb2 = res.F; beta2_b = res.beta; Lb2=res.L;

    J2b = test2(X,beta2_b,eb2,Lb2,Fb2,h,kernelType);

    %% generate data for H3: allow for TV slope


    Y3 = sum(X.*repmat(beta,N,1),2) + C_ + ee;
    res = kpanelife(Y3,X,T,R,h,kernelType,1,XTol,maxIter);
    eb = res.e; Fb = res.F; Cb=res.C; 
    [~,C3b,F3b] = panelife(Y3,X,T,R,XTol,maxIter);
    J3b = test3(Cb,Fb,eb,C3b,F3b,h,kernelType); 
  
    J(i,:)=[J1b, J2b, J3b ];
end
p1 = sum(J(:,1)>J1)/nb;

p2 = sum(J(:,2)>J2)/nb;

p3 = sum(J(:,3)>J3)/nb;

 
 