function [Y,X,beta,C,F,lambda,e]=dgp(N,T,mtype,etype,sigma)
% C is the true common component 
if nargin < 5 || isempty(sigma)
    sigma = 1;
end

X1 = ar1rnd(0.7,sqrt(1-0.7^2),T,N);
X2 = ar1rnd(0.4,sqrt(1-0.4^2),T,N);

F1 = ar1rnd(0.6,sqrt(1-0.6^2),T);
F2 = ar1rnd(0.3,sqrt(1-0.3^2),T);
F = [F1 F2];

switch etype
    case 1
        e = sigma*randn(T,N);
    case 2
        s = rand(1,N)+0.5;
        e = sigma*repmat(s,T,1).*randn(T,N);
    case 3
        rho = 0.2+rand(1,N)*0.4; 
        e = sigma*ar1rnd(rho,sqrt(1-rho.^2),T,N);
        e = e-ones(T,1)*mean(e); 
end


switch mtype
    case 1
        %% DGP 1: Time-invariant slope and factor loadings, X indep. F  
        beta1 = 0.75;
        beta2 = 0.25;
        lambda = randn(N,2);
        C = F*(lambda');
        Y = X1*beta1 + X2*beta2 + C + e;
        beta = repmat([beta1 beta2],T,1);
    case 2
        %% DGP2 time-invariant slope, TV factor loadings, X indep F 
        beta1 = 0.75;
        beta2 = 0.25;
        lambda1 = repmat(randn(1,N),T,1);
        lambda2 = zeros(T,N);
        for i=1:N
            lambda2(:,i) = cos(pi*(1:T)'/T+i/N*pi);
        end
        C = lambda1.*repmat(F1,1,N) + lambda2.*repmat(F2,1,N);
        Y = X1*beta1 + X2*beta2 + C + e;
        lambda = [reshape(lambda1,N*T,1) reshape(lambda2,N*T,1)];
        beta = repmat([beta1 beta2],T,1);
    case 3
        %% time-invariant slope, TV factor loadings, X depend on F 
        beta1 = 0.75;
        beta2 = 0.25;
        lambda1 = zeros(T,N);
        lambda2 = repmat(randn(1,N),T,1);
        for i=1:N
            lambda1(:,i) = cos(pi*(1:T)'/T+i/N*pi);
        end
        C = lambda1.*repmat(F1,1,N) + lambda2.*repmat(F2,1,N);
        X1 = 0.5*randn(T,N) + 0.5*[F1 F2]*rand(2,N);
        X2 = 0.5*randn(T,N) + 0.5*[F1 F2]*rand(2,N);
        Y = X1*beta1 + X2*beta2 + C + e;
        lambda = [reshape(lambda1,N*T,1) reshape(lambda2,N*T,1)];
        beta = repmat([beta1 beta2],T,1);
    case 4
        %% TV slope, time-invariant factor loadings, X indep F 
        beta1 = sin(pi/2*(1:T)'/T);
        beta2 = 2*((1:T)'/T-0.8).^2;
        lambda1 = repmat(randn(1,N),T,1);
        lambda2 = repmat(randn(1,N),T,1);
        C = lambda1.*repmat(F1,1,N) + lambda2.*repmat(F2,1,N);
        Y = X1.*repmat(beta1,1,N) + X2.*repmat(beta2,1,N) + C + e;
        lambda = [reshape(lambda1,N*T,1) reshape(lambda2,N*T,1)];
        beta = [beta1 beta2];
    case 5
        beta1 = sin(pi/2*(1:T)'/T);
        beta2 = 2*((1:T)'/T-0.8).^2;
        lambda1 = repmat(randn(1,N),T,1);
        lambda2 = zeros(T,N);
        for i=1:N 
            lambda2(:,i) = cos(pi*(1:T)'/T+i/N*pi);
        end
        C = lambda1.*repmat(F1,1,N) + lambda2.*repmat(F2,1,N);
        Y = X1.*repmat(beta1,1,N) + X2.*repmat(beta2,1,N) + C + e;
        lambda = [reshape(lambda1,N*T,1) reshape(lambda2,N*T,1)];
        beta = [beta1 beta2];
    case 6
        beta1 = sin(pi/2*(1:T)'/T);
        beta2 = 2*((1:T)'/T-0.8).^2;
        lambda2 = repmat(randn(1,N),T,1);
        lambda1 = zeros(T,N);
        for i=1:N 
            lambda1(:,i) = cos(pi*(1:T)'/T+i/N*pi);
        end
        C = lambda1.*repmat(F1,1,N) + lambda2.*repmat(F2,1,N);
        X1 = 0.5*randn(T,N) + 0.5*[F1 F2]*rand(2,N);
        X2 = 0.5*randn(T,N) + 0.5*[F1 F2]*rand(2,N);
        Y = X1.*repmat(beta1,1,N) + X2.*repmat(beta2,1,N) + C + e;
        lambda = [reshape(lambda1,N*T,1) reshape(lambda2,N*T,1)];
        beta = [beta1 beta2];
end

Y = reshape(Y,N*T,1);
X = [reshape(X1,N*T,1) reshape(X2,N*T,1)];
C = reshape(C,N*T,1);
e = reshape(e,N*T,1);
