function [beta,F,L,flag] = kpanelife_(Y,X,T,R,r,h,kernelType,boundaryKern,XTol,maxIter)

[n,p] = size(X);
N = n/T;
lr = length(r);

if nargin < 10 || isempty(maxIter)
    maxIter = 2000*p;
end
if nargin < 9 || isempty(XTol)
    XTol = 1e-4; 
end
if nargin < 8 || isempty(boundaryKern)
    boundaryKern=1;
end
if nargin < 7 || isempty(kernelType)
    kernelType=1;
end
if nargin < 6 || isempty(h)
    h=2.35/sqrt(12)*T^(-1/5)*N^(-1/10);
end
if nargin < 5 || isempty(r)
    r=(1:T)';
end

if boundaryKern
    kh = @khtr_b;
else
    kh = @khtr;
end
flag = 1;
beta = zeros(lr,p);
L = zeros(lr,N*R);
F = zeros(lr,T*R);
for i=1:lr
    rr = r(i);
    w = kh(h,(1:T)',rr,T,kernelType).^(1/2); % T-by-1
    if sum(w>0)<(p+1)
        disp('bandwidth too small');
        beta = [];
        F = [];
        L = [];
        flag = 0;
        return;
    end
    Y_ = Y.*repmat(w,N,1);
    X_ = X.*repmat(w,N,p);
    [br,~,Fr,Lr,fl] = panelife(Y_,X_,T,R,XTol,maxIter);
    beta(i,:) = br';    % 1 x p 
    L(i,:) = reshape(Lr,N*R,1)';   % 1 x NR 
    F(i,:) = reshape(Fr,T*R,1)';
    flag = flag * fl;
end
if flag==0
    disp('panelife fails to converge at some point. You may increase maxIter.');
end


